@@ -7,13 +7,15 @@
 use {
     crate::{
         code_directory::CodeDirectoryBlob,
+        code_requirement::RequirementType,
         code_resources::{CodeResourcesBuilder, CodeResourcesRule},
+        embedded_signature::{Blob, BlobData, CodeSigningSlot},
         error::AppleCodesignError,
-        macho::{AppleSignable, Blob, BlobData, CodeSigningSlot, RequirementType},
+        macho::AppleSignable,
         macho_signing::MachOSigner,
-        signing::{SettingsScope, SigningSettings},
+        signing_settings::{SettingsScope, SigningSettings},
     },
-    apple_bundles::{DirectoryBundle, DirectoryBundleFile},
+    apple_bundles::{BundlePackageType, DirectoryBundle, DirectoryBundleFile},
     goblin::mach::Mach,
     log::{info, warn},
     std::{
@@ -21,8 +23,28 @@ use {
         io::Write,
         path::{Path, PathBuf},
     },
+    tugger_file_manifest::create_symlink,
 };
 
+/// Copy a bundle's contents to a destination directory.
+pub fn copy_bundle(bundle: &DirectoryBundle, dest_dir: &Path) -> Result<(), AppleCodesignError> {
+    let settings = SigningSettings::default();
+
+    let handler = SingleBundleHandler {
+        dest_dir: dest_dir.to_path_buf(),
+        settings: &settings,
+    };
+
+    for file in bundle
+        .files(false)
+        .map_err(AppleCodesignError::DirectoryBundle)?
+    {
+        handler.install_file(&file)?;
+    }
+
+    Ok(())
+}
+
 /// A primitive for signing an Apple bundle.
 ///
 /// This type handles the high-level logic of signing an Apple bundle (e.g.
@@ -44,7 +66,7 @@ impl BundleSigner {
             .map_err(AppleCodesignError::DirectoryBundle)?;
 
         let mut bundles = main_bundle
-            .nested_bundles()
+            .nested_bundles(true)
             .map_err(AppleCodesignError::DirectoryBundle)?
             .into_iter()
             .map(|(k, bundle)| (Some(k), SingleBundleSigner::new(bundle)))
@@ -67,47 +89,52 @@ impl BundleSigner {
     ) -> Result<DirectoryBundle, AppleCodesignError> {
         let dest_dir = dest_dir.as_ref();
 
-        let mut additional_files = Vec::new();
+        // We need to sign the leaf-most bundles first since a parent bundle may need
+        // to record information about the child in its signature.
+        let mut bundles = self
+            .bundles
+            .iter()
+            .filter_map(|(rel, bundle)| rel.as_ref().map(|rel| (rel, bundle)))
+            .collect::<Vec<_>>();
 
-        for (rel, nested) in &self.bundles {
-            match rel {
-                Some(rel) => {
-                    let nested_dest_dir = dest_dir.join(rel);
-                    info!(
-                        "entering nested bundle {}",
-                        nested.bundle.root_dir().display(),
-                    );
-                    let signed_bundle = nested.write_signed_bundle(
-                        nested_dest_dir,
-                        &settings.as_nested_bundle_settings(rel),
-                        &[],
-                    )?;
+        // This won't preserve alphabetical order. But since the input was stable, output
+        // should be deterministic.
+        bundles.sort_by(|(a, _), (b, _)| b.len().cmp(&a.len()));
 
-                    // The main bundle's CodeResources file contains references to metadata about
-                    // nested bundles' main executables. So we capture that here.
-                    let main_exe = signed_bundle
-                        .files(false)
-                        .map_err(AppleCodesignError::DirectoryBundle)?
-                        .into_iter()
-                        .find(|file| matches!(file.is_main_executable(), Ok(true)));
+        warn!(
+            "signing {} nested bundles in the following order:",
+            bundles.len()
+        );
+        for bundle in &bundles {
+            warn!("{}", bundle.0);
+        }
 
-                    if let Some(main_exe) = main_exe {
-                        let macho_data = std::fs::read(main_exe.absolute_path())?;
-                        let macho_info = SignedMachOInfo::parse_data(&macho_data)?;
+        for (rel, nested) in bundles {
+            let nested_dest_dir = dest_dir.join(rel);
+            info!(
+                "entering nested bundle {}",
+                nested.bundle.root_dir().display(),
+            );
 
-                        let path = rel.replace('\\', "/");
-                        let path = path.strip_prefix("Contents/").unwrap_or(&path).to_string();
-
-                        additional_files.push((path, macho_info));
-                    }
-
-                    info!(
-                        "leaving nested bundle {}",
-                        nested.bundle.root_dir().display()
-                    );
-                }
-                None => {}
+            // If we excluded this bundle from signing, just copy all the files.
+            if settings
+                .path_exclusion_patterns()
+                .iter()
+                .any(|pattern| pattern.matches(rel))
+            {
+                warn!("bundle is in exclusion list; it will be copied instead of signed");
+                copy_bundle(&nested.bundle, &nested_dest_dir)?;
+            } else {
+                nested.write_signed_bundle(
+                    nested_dest_dir,
+                    &settings.as_nested_bundle_settings(rel),
+                )?;
             }
+
+            info!(
+                "leaving nested bundle {}",
+                nested.bundle.root_dir().display()
+            );
         }
 
         let main = self
@@ -115,7 +142,7 @@ impl BundleSigner {
             .get(&None)
             .expect("main bundle should have a key");
 
-        main.write_signed_bundle(dest_dir, settings, &additional_files)
+        main.write_signed_bundle(dest_dir, settings)
     }
 }
 
@@ -193,9 +220,9 @@ impl SignedMachOInfo {
     pub fn notarization_ticket_record_name(&self) -> Result<String, AppleCodesignError> {
         let cd = self.code_directory()?;
 
-        let digest_type: u8 = cd.hash_type.into();
+        let digest_type: u8 = cd.digest_type.into();
 
-        let mut digest = cd.digest_with(cd.hash_type)?;
+        let mut digest = cd.digest_with(cd.digest_type)?;
 
         // Digests appear to be truncated at 20 bytes / 40 characters.
         digest.truncate(20);
@@ -236,17 +263,30 @@ impl<'a, 'key> BundleFileHandler for SingleBundleHandler<'a, 'key> {
         let dest_path = self.dest_dir.join(file.relative_path());
 
         if source_path != dest_path {
-            info!(
-                "copying file {} -> {}",
-                source_path.display(),
-                dest_path.display()
-            );
             std::fs::create_dir_all(
                 dest_path
                     .parent()
                     .expect("parent directory should be available"),
             )?;
-            std::fs::copy(source_path, dest_path)?;
+
+            if let Some(target) = file
+                .symlink_target()
+                .map_err(AppleCodesignError::DirectoryBundle)?
+            {
+                info!(
+                    "replicating symlink {} -> {}",
+                    dest_path.display(),
+                    target.display()
+                );
+                create_symlink(&dest_path, target)?;
+            } else {
+                info!(
+                    "copying file {} -> {}",
+                    source_path.display(),
+                    dest_path.display()
+                );
+                std::fs::copy(source_path, dest_path)?;
+            }
         }
 
         Ok(())
@@ -265,19 +305,7 @@ impl<'a, 'key> BundleFileHandler for SingleBundleHandler<'a, 'key> {
             .settings
             .as_bundle_macho_settings(file.relative_path().to_string_lossy().as_ref());
 
-        // The identifier string for a Mach-O that isn't the main executable is the
-        // file name, without a `.dylib` extension.
-        // TODO consider adding logic to SigningSettings?
-        let identifier = file
-            .relative_path()
-            .file_name()
-            .expect("failure to extract filename (this should never happen)")
-            .to_string_lossy();
-
-        let identifier = identifier
-            .strip_suffix(".dylib")
-            .unwrap_or_else(|| identifier.as_ref());
-        settings.set_binary_identifier(SettingsScope::Main, identifier);
+        settings.import_settings_from_macho(&macho_data)?;
 
         let mut new_data = Vec::<u8>::with_capacity(macho_data.len() + 2_usize.pow(17));
         signer.write_signed_binary(&settings, &mut new_data)?;
@@ -323,7 +351,6 @@ impl SingleBundleSigner {
         &self,
         dest_dir: impl AsRef<Path>,
         settings: &SigningSettings,
-        additional_macho_files: &[(String, SignedMachOInfo)],
     ) -> Result<DirectoryBundle, AppleCodesignError> {
         let dest_dir = dest_dir.as_ref();
 
@@ -333,6 +360,47 @@ impl SingleBundleSigner {
             dest_dir.display()
         );
 
+        // Frameworks are a bit special.
+        //
+        // Modern frameworks typically have a `Versions/` directory containing directories
+        // with the actual frameworks. These are the actual directories that are signed - not
+        // the top-most directory. In fact, the top-most `.framework` directory doesn't have any
+        // code signature elements at all and can effectively be ignored as far as signing
+        // is concerned.
+        //
+        // But even if there is a `Versions/` directory with nested bundles to sign, the top-level
+        // directory may have some symlinks. And those need to be preserved. In addition, there
+        // may be symlinks in `Versions/`. `Versions/Current` is common.
+        //
+        // Of course, if there is no `Versions/` directory, the top-level directory could be
+        // a valid framework warranting signing.
+        if self.bundle.package_type() == BundlePackageType::Framework {
+            if self.bundle.root_dir().join("Versions").is_dir() {
+                warn!("found a versioned framework; each version will be signed as its own bundle");
+
+                // But we still need to preserve files (hopefully just symlinks) outside the
+                // nested bundles under `Versions/`. Since we don't nest into child bundles
+                // here, it should be safe to handle each encountered file.
+                let handler = SingleBundleHandler {
+                    dest_dir: dest_dir.to_path_buf(),
+                    settings,
+                };
+
+                for file in self
+                    .bundle
+                    .files(false)
+                    .map_err(AppleCodesignError::DirectoryBundle)?
+                {
+                    handler.install_file(&file)?;
+                }
+
+                return DirectoryBundle::new_from_path(dest_dir)
+                    .map_err(AppleCodesignError::DirectoryBundle);
+            } else {
+                warn!("found an unversioned framework; signing like normal");
+            }
+        }
+
         let dest_dir_root = dest_dir.to_path_buf();
 
         let dest_dir = if self.bundle.shallow() {
@@ -347,7 +415,23 @@ impl SingleBundleSigner {
             .ok_or_else(|| AppleCodesignError::BundleNoIdentifier(self.bundle.info_plist_path()))?;
 
         warn!("collecting code resources files");
-        let mut resources_builder = CodeResourcesBuilder::default_resources_rules()?;
+
+        // The set of rules to use is determined by whether the bundle *can* have a
+        // `Resources/`, not whether it necessarily does. The exact rules for this are not
+        // known. Essentially we want to test for the result of CFBundleCopyResourcesDirectoryURL().
+        // We assume that we can use the resources rules when there is a `Resources` directory
+        // (this seems obvious!) or when the bundle isn't shallow, as a non-shallow bundle should
+        // be an app bundle and app bundles can always have resources (we think).
+        let mut resources_builder =
+            if self.bundle.resolve_path("Resources").is_dir() || !self.bundle.shallow() {
+                CodeResourcesBuilder::default_resources_rules()?
+            } else {
+                CodeResourcesBuilder::default_no_resources_rules()?
+            };
+
+        // Ensure emitted digests match what we're configured to emit.
+        resources_builder.set_digests(settings.all_digests(SettingsScope::Main).into_iter());
+
         // Exclude code signature files we'll write.
         resources_builder.add_exclusion_rule(CodeResourcesRule::new("^_CodeSignature/")?.exclude());
         // Ignore notarization ticket.
@@ -363,10 +447,17 @@ impl SingleBundleSigner {
 
         // Iterate files in this bundle and register as code resources.
         //
-        // Encountered Mach-O binaries will need to be signed.
+        // Traversing into nested bundles seems wrong but it is correct. The resources builder
+        // has rules to determine whether to process a path and assuming the rules and evaluation
+        // of them is correct, it is able to decide for itself how to handle a path.
+        //
+        // Furthermore, this behavior is needed as bundles can encapsulate signatures for nested
+        // bundles. For example, you could have a framework bundle with an embedded app bundle in
+        // `Resources/MyApp.app`! In this case, the framework's CodeResources encapsulates the
+        // content of `Resources/My.app` per the processing rules.
         for file in self
             .bundle
-            .files(false)
+            .files(true)
             .map_err(AppleCodesignError::DirectoryBundle)?
         {
             // The main executable is special and handled below.
@@ -374,20 +465,41 @@ impl SingleBundleSigner {
                 .is_main_executable()
                 .map_err(AppleCodesignError::DirectoryBundle)?
             {
+                info!(
+                    "{} is identified as the main executable; saving it for later",
+                    file.relative_path().display()
+                );
                 main_exe = Some(file);
-            // The Info.plist is digested specially.
             } else if file.is_info_plist() {
-                handler.install_file(&file)?;
+                // The Info.plist is digested specially. But it may also be handled by
+                // the resources handler. So always feed it through.
+                info!(
+                    "{} is the Info.plist file; handling specially",
+                    file.relative_path().display()
+                );
+                resources_builder.process_file(&file, &handler)?;
                 info_plist_data = Some(std::fs::read(file.absolute_path())?);
             } else {
                 resources_builder.process_file(&file, &handler)?;
             }
         }
 
-        // Add in any additional signed Mach-O files. This is likely used for nested
-        // bundles.
-        for (path, info) in additional_macho_files {
-            resources_builder.add_signed_macho_file(path, info)?;
+        // Seal code directory digests of any nested bundles.
+        //
+        // Apple's tooling seems to only do this for some bundle type combinations. I'm
+        // not yet sure what the complete heuristic is. But we observed that frameworks
+        // don't appear to include digests of any nested app bundles. So we add that
+        // exclusion. We should figure out what the actual rules here...
+        if self.bundle.package_type() != BundlePackageType::Framework {
+            let dest_bundle = DirectoryBundle::new_from_path(&dest_dir)
+                .map_err(AppleCodesignError::DirectoryBundle)?;
+
+            for (rel_path, nested_bundle) in dest_bundle
+                .nested_bundles(false)
+                .map_err(AppleCodesignError::DirectoryBundle)?
+            {
+                resources_builder.process_nested_bundle(&rel_path, &nested_bundle)?;
+            }
         }
 
         // The resources are now sealed. Write out that XML file.
@@ -414,13 +526,18 @@ impl SingleBundleSigner {
 
             let mut settings = settings.clone();
 
+            settings.import_settings_from_macho(&macho_data)?;
+
             // The identifier for the main executable is defined in the bundle's Info.plist.
             if let Some(ident) = self
                 .bundle
                 .identifier()
                 .map_err(AppleCodesignError::DirectoryBundle)?
             {
+                info!("setting main executable binary identifier to {} (derived from CFBundleIdentifier in Info.plist)", ident);
                 settings.set_binary_identifier(SettingsScope::Main, ident);
+            } else {
+                info!("unable to determine binary identifier from bundle's Info.plist (CFBundleIdentifier not set?)");
             }
 
             settings.set_code_resources_data(SettingsScope::Main, resources_data);
@@ -433,6 +550,7 @@ impl SingleBundleSigner {
             signer.write_signed_binary(&settings, &mut new_data)?;
 
             let dest_path = dest_dir_root.join(exe.relative_path());
+            info!("writing signed main executable to {}", dest_path.display());
 
             let permissions = std::fs::metadata(exe.absolute_path())?.permissions();
             std::fs::create_dir_all(
@@ -445,6 +563,8 @@ impl SingleBundleSigner {
                 fh.write_all(&new_data)?;
             }
             std::fs::set_permissions(&dest_path, permissions)?;
+        } else {
+            warn!("bundle has no main executable to sign specially");
         }
 
         DirectoryBundle::new_from_path(&dest_dir_root).map_err(AppleCodesignError::DirectoryBundle)
