@@ -18,13 +18,18 @@
 use {
     crate::{
         bundle_signing::{BundleFileHandler, SignedMachOInfo},
+        embedded_signature::DigestType,
         error::AppleCodesignError,
-        macho::DigestType,
     },
-    apple_bundles::DirectoryBundleFile,
-    log::{debug, info},
+    apple_bundles::{DirectoryBundle, DirectoryBundleFile},
+    log::{debug, info, warn},
     plist::{Dictionary, Value},
-    std::{cmp::Ordering, collections::BTreeMap, io::Write},
+    std::{
+        cmp::Ordering,
+        collections::BTreeMap,
+        io::Write,
+        path::{Path, PathBuf},
+    },
 };
 
 #[derive(Clone, PartialEq)]
@@ -138,6 +143,7 @@ impl From<&FilesValue> for Value {
 #[derive(Clone, PartialEq)]
 struct Files2Value {
     cdhash: Option<Vec<u8>>,
+    hash: Option<Vec<u8>>,
     hash2: Option<Vec<u8>>,
     optional: Option<bool>,
     requirement: Option<String>,
@@ -151,6 +157,10 @@ impl std::fmt::Debug for Files2Value {
                 "cdhash",
                 &format_args!("{:?}", self.cdhash.as_ref().map(hex::encode)),
             )
+            .field(
+                "hash",
+                &format_args!("{:?}", self.hash.as_ref().map(hex::encode)),
+            )
             .field(
                 "hash2",
                 &format_args!("{:?}", self.hash2.as_ref().map(hex::encode)),
@@ -170,6 +180,7 @@ impl TryFrom<&Value> for Files2Value {
             AppleCodesignError::ResourcesPlistParse("files2 value should be a dict".to_string())
         })?;
 
+        let mut hash = None;
         let mut hash2 = None;
         let mut cdhash = None;
         let mut optional = None;
@@ -188,7 +199,7 @@ impl TryFrom<&Value> for Files2Value {
 
                     cdhash = Some(data.to_vec());
                 }
-                "hash2" => {
+                "hash" => {
                     let data = value.as_data().ok_or_else(|| {
                         AppleCodesignError::ResourcesPlistParse(format!(
                             "expected <data> for files2 hash entry, got {:?}",
@@ -196,6 +207,16 @@ impl TryFrom<&Value> for Files2Value {
                         ))
                     })?;
 
+                    hash = Some(data.to_vec());
+                }
+                "hash2" => {
+                    let data = value.as_data().ok_or_else(|| {
+                        AppleCodesignError::ResourcesPlistParse(format!(
+                            "expected <data> for files2 hash2 entry, got {:?}",
+                            value
+                        ))
+                    })?;
+
                     hash2 = Some(data.to_vec());
                 }
                 "optional" => {
@@ -242,6 +263,7 @@ impl TryFrom<&Value> for Files2Value {
 
         Ok(Self {
             cdhash,
+            hash,
             hash2,
             optional,
             requirement,
@@ -258,6 +280,10 @@ impl From<&Files2Value> for Value {
             dict.insert("cdhash".to_string(), Value::Data(cdhash.to_vec()));
         }
 
+        if let Some(hash) = &v.hash {
+            dict.insert("hash".to_string(), Value::Data(hash.to_vec()));
+        }
+
         if let Some(hash2) = &v.hash2 {
             dict.insert("hash2".to_string(), Value::Data(hash2.to_vec()));
         }
@@ -590,6 +616,17 @@ impl CodeResourcesRule {
     }
 }
 
+/// Which files section we are operating on and how to digest.
+#[derive(Clone, Copy, Debug)]
+pub enum FilesFlavor {
+    /// `<rules>`.
+    Rules,
+    /// `<rules2>`.
+    Rules2,
+    /// `<rules2>` and also include the SHA-1 digest.
+    Rules2WithSha1,
+}
+
 /// Represents a `_CodeSignature/CodeResources` XML plist.
 ///
 /// This file/type represents a collection of file-based resources whose
@@ -735,41 +772,68 @@ impl CodeResources {
 
     /// Seal a regular file.
     ///
-    /// This will digest the content specified and record that digest in the files list.
+    /// This will digest the content specified and record that digest in the files or
+    /// files2 list.
     ///
     /// To seal a symlink, call [CodeResources::seal_symlink] instead. If the file
     /// is a Mach-O file, call [CodeResources::seal_macho] instead.
     pub fn seal_regular_file(
         &mut self,
+        files_flavor: FilesFlavor,
         path: impl ToString,
         content: impl AsRef<[u8]>,
         optional: bool,
     ) -> Result<(), AppleCodesignError> {
-        let sha1 = DigestType::Sha1.digest(content.as_ref())?;
-        let sha256 = DigestType::Sha256.digest(content.as_ref())?;
+        match files_flavor {
+            FilesFlavor::Rules => {
+                let digest = DigestType::Sha1.digest_data(content.as_ref())?;
+                self.files.insert(
+                    path.to_string(),
+                    if optional {
+                        FilesValue::Optional(digest)
+                    } else {
+                        FilesValue::Required(digest)
+                    },
+                );
 
-        let path = path.to_string();
+                Ok(())
+            }
+            FilesFlavor::Rules2 => {
+                let hash2 = Some(DigestType::Sha256.digest_data(content.as_ref())?);
 
-        self.files.insert(
-            path.clone(),
-            if optional {
-                FilesValue::Optional(sha1)
-            } else {
-                FilesValue::Required(sha1)
-            },
-        );
-        self.files2.insert(
-            path,
-            Files2Value {
-                cdhash: None,
-                hash2: Some(sha256),
-                optional: if optional { Some(true) } else { None },
-                requirement: None,
-                symlink: None,
-            },
-        );
+                self.files2.insert(
+                    path.to_string(),
+                    Files2Value {
+                        cdhash: None,
+                        hash: None,
+                        hash2,
+                        optional: if optional { Some(true) } else { None },
+                        requirement: None,
+                        symlink: None,
+                    },
+                );
 
-        Ok(())
+                Ok(())
+            }
+            FilesFlavor::Rules2WithSha1 => {
+                let hash = Some(DigestType::Sha1.digest_data(content.as_ref())?);
+                let hash2 = Some(DigestType::Sha256.digest_data(content.as_ref())?);
+
+                self.files2.insert(
+                    path.to_string(),
+                    Files2Value {
+                        cdhash: None,
+                        hash,
+                        hash2,
+                        optional: if optional { Some(true) } else { None },
+                        requirement: None,
+                        symlink: None,
+                    },
+                );
+
+                Ok(())
+            }
+        }
     }
 
     /// Seal a symlink file.
@@ -780,6 +844,7 @@ impl CodeResources {
             path.to_string(),
             Files2Value {
                 cdhash: None,
+                hash: None,
                 hash2: None,
                 optional: None,
                 requirement: None,
@@ -800,7 +865,8 @@ impl CodeResources {
         self.files2.insert(
             path.to_string(),
             Files2Value {
-                cdhash: Some(DigestType::Sha256Truncated.digest(&info.code_directory_blob)?),
+                cdhash: Some(DigestType::Sha256Truncated.digest_data(&info.code_directory_blob)?),
+                hash: None,
                 hash2: None,
                 optional: if optional { Some(true) } else { None },
                 requirement: info.designated_code_requirement.clone(),
@@ -864,15 +930,55 @@ impl From<&CodeResources> for Value {
     }
 }
 
+#[derive(Clone, Debug)]
+enum RulesEvaluation {
+    /// File should be ignored completely.
+    Exclude,
+
+    /// File isn't sealed but it is installed.
+    Omit,
+
+    /// Seal a symlink.
+    ///
+    /// Members are the relative path and the target path.
+    SealSymlink(String, String),
+
+    /// Seal a nested Mach-O binary.
+    ///
+    /// Members are the relative path and whether the rule is optional.
+    SealNested(String, bool),
+
+    /// Seal a regular file.
+    ///
+    /// Members are the relative path and whether the rule is optional.
+    SealRegularFile(String, bool),
+
+    /// File doesn't match any rules.
+    NoRule,
+}
+
 /// Interface for constructing a `CodeResources` instance.
 ///
 /// This type is used during bundle signing to construct a `CodeResources` instance.
 /// It contains logic for validating a file against registered processing rules and
 /// handling it accordingly.
-#[derive(Clone, Debug, Default)]
+#[derive(Clone, Debug)]
 pub struct CodeResourcesBuilder {
     rules: Vec<CodeResourcesRule>,
+    rules2: Vec<CodeResourcesRule>,
     resources: CodeResources,
+    digests: Vec<DigestType>,
+}
+
+impl Default for CodeResourcesBuilder {
+    fn default() -> Self {
+        Self {
+            rules: vec![],
+            rules2: vec![],
+            resources: CodeResources::default(),
+            digests: vec![DigestType::Sha256],
+        }
+    }
 }
 
 impl CodeResourcesBuilder {
@@ -967,16 +1073,22 @@ impl CodeResourcesBuilder {
         Ok(slf)
     }
 
+    /// Set the digests to record in this instance.
+    pub fn set_digests(&mut self, digests: impl Iterator<Item = DigestType>) {
+        self.digests = digests.collect::<Vec<_>>();
+    }
+
     /// Add a rule to this instance in the `<rules>` section.
     pub fn add_rule(&mut self, rule: CodeResourcesRule) {
-        // Don't set internal rules because we only operate in a v2 world.
+        self.rules.push(rule.clone());
+        self.rules.sort();
         self.resources.add_rule(rule);
     }
 
     /// Add a rule to this instance in the `<rules2>` section.
     pub fn add_rule2(&mut self, rule: CodeResourcesRule) {
-        self.rules.push(rule.clone());
-        self.rules.sort();
+        self.rules2.push(rule.clone());
+        self.rules2.sort();
         self.resources.add_rule2(rule);
     }
 
@@ -985,8 +1097,10 @@ impl CodeResourcesBuilder {
     /// Exclusion rules are not added to the [CodeResources] because they are
     /// for building only.
     pub fn add_exclusion_rule(&mut self, rule: CodeResourcesRule) {
-        self.rules.push(rule);
+        self.rules.push(rule.clone());
         self.rules.sort();
+        self.rules2.push(rule);
+        self.rules2.sort();
     }
 
     /// Find the first rule matching a given path.
@@ -1000,15 +1114,13 @@ impl CodeResourcesBuilder {
     ///
     /// Directories are special. If an exclusion rule matches a directory, that directory
     /// tree should be ignored. There are also default rules for handling nested bundles.
-    /// However, our bundle scanning code already filters out nested bundles automatically,
-    /// so these rules shouldn't be relevant to us. But we handle them anyway, just in
-    /// case. These rules take precedence over directory exclusion rules.
-    fn find_rule(&self, path: &str) -> Option<CodeResourcesRule> {
+    /// These rules take precedence over directory exclusion rules.
+    fn find_rule(rules: &[CodeResourcesRule], path: &str) -> Option<CodeResourcesRule> {
         let parts = path.split('/').collect::<Vec<_>>();
 
         let mut exclude_override = false;
 
-        let rule = self.rules.iter().find(|rule| {
+        let rule = rules.iter().find(|rule| {
             // Nested rules matching leaf-most directory with `.` result in match.
             // But we treat as exclusion, as these are treated as nested bundles,
             // which are handled externally.
@@ -1051,17 +1163,13 @@ impl CodeResourcesBuilder {
         }
     }
 
-    /// Process a file for resource handling.
-    ///
-    /// This determines whether a file is relevant for inclusion in the CodeResources
-    /// file and takes actions to process it, if necessary.
-    pub fn process_file(
-        &mut self,
-        file: &DirectoryBundleFile,
-        file_handler: &dyn BundleFileHandler,
-    ) -> Result<(), AppleCodesignError> {
+    fn evaluate_rules(
+        rules: &[CodeResourcesRule],
+        relative_path: impl AsRef<Path>,
+        symlink_target: Option<PathBuf>,
+    ) -> Result<RulesEvaluation, AppleCodesignError> {
         // Always use UNIX style directory separators.
-        let relative_path = file.relative_path().to_string_lossy().replace('\\', "/");
+        let relative_path = relative_path.as_ref().to_string_lossy().replace('\\', "/");
 
         // The Contents/ prefix is also removed for pattern matching and references in the
         // resources file.
@@ -1070,9 +1178,7 @@ impl CodeResourcesBuilder {
             .unwrap_or(&relative_path)
             .to_string();
 
-        info!("processing {}", relative_path);
-
-        let rule = match self.find_rule(relative_path.as_ref()) {
+        match Self::find_rule(rules, relative_path.as_ref()) {
             Some(rule) => {
                 debug!(
                     "{} matches {} rule {}",
@@ -1085,64 +1191,184 @@ impl CodeResourcesBuilder {
                     rule.pattern
                 );
 
-                // Excluded files are hard ignored (our caller will handle them if necessary).
                 if rule.exclude {
-                    return Ok(());
-                // Omitted files aren't sealed. But they are installed.
+                    Ok(RulesEvaluation::Exclude)
                 } else if rule.omit {
-                    return file_handler.install_file(file);
-                }
+                    Ok(RulesEvaluation::Omit)
+                } else if rule.nested && symlink_target.is_some() {
+                    // Symlinks in nested bundles can be excluded since they should have
+                    // been processed by the nested bundle.
+                    Ok(RulesEvaluation::Exclude)
+                } else if let Some(target) = symlink_target {
+                    let target = target.to_string_lossy().replace('\\', "/");
 
-                rule
+                    Ok(RulesEvaluation::SealSymlink(relative_path, target))
+                } else if rule.nested {
+                    Ok(RulesEvaluation::SealNested(relative_path, rule.optional))
+                } else {
+                    Ok(RulesEvaluation::SealRegularFile(
+                        relative_path,
+                        rule.optional,
+                    ))
+                }
             }
             None => {
-                debug!("{} doesn't match any rule; processing", relative_path);
+                debug!("{} doesn't match any rule", relative_path);
+                Ok(RulesEvaluation::NoRule)
+            }
+        }
+    }
+
+    /// Process the `<rules2>` set for a given file.
+    fn process_file_rules2(
+        &mut self,
+        file: &DirectoryBundleFile,
+        file_handler: &dyn BundleFileHandler,
+    ) -> Result<(), AppleCodesignError> {
+        match Self::evaluate_rules(
+            &self.rules2,
+            file.relative_path(),
+            file.symlink_target()
+                .map_err(AppleCodesignError::DirectoryBundle)?,
+        )? {
+            RulesEvaluation::Exclude => {
+                // Excluded files are hard ignored. These files are likely handled out-of-band
+                // from this builder.
+                Ok(())
+            }
+            RulesEvaluation::Omit => {
+                // Omitted files aren't sealed. But they are installed.
+                file_handler.install_file(file)
+            }
+            RulesEvaluation::NoRule => {
+                // No rule match is assumed to mean full ignore.
+                Ok(())
+            }
+            RulesEvaluation::SealSymlink(relative_path, target) => {
+                info!("sealing symlink {} -> {}", relative_path, target);
+                self.resources.seal_symlink(relative_path, target);
+                file_handler.install_file(file)
+            }
+            RulesEvaluation::SealNested(relative_path, optional) => {
+                // The assumption that a nested match means Mach-O may not be correct.
+                info!("sealing Mach-O file {}", relative_path);
+                let macho_info = file_handler.sign_and_install_macho(file)?;
+
+                self.resources
+                    .seal_macho(relative_path, &macho_info, optional)
+            }
+            RulesEvaluation::SealRegularFile(relative_path, optional) => {
+                info!("sealing regular file {}", relative_path);
+                let data = std::fs::read(file.absolute_path())?;
+
+                let flavor = if self.digests.contains(&DigestType::Sha1) {
+                    FilesFlavor::Rules2WithSha1
+                } else {
+                    FilesFlavor::Rules2
+                };
+
+                self.resources
+                    .seal_regular_file(flavor, relative_path, data, optional)?;
+                file_handler.install_file(file)
+            }
+        }
+    }
+
+    /// Process the `<rules>` set for a given file.
+    ///
+    /// Since `<rules2>` handling actually does the file installs, the only role of this
+    /// handler is to record the SHA-1 seals in `<files>`. Keep in mind that `<files>` can't
+    /// handle symlinks or nested Mach-O binaries. So we only care about regular files here.
+    fn process_file_rules(&mut self, file: &DirectoryBundleFile) -> Result<(), AppleCodesignError> {
+        match Self::evaluate_rules(
+            &self.rules,
+            file.relative_path(),
+            file.symlink_target()
+                .map_err(AppleCodesignError::DirectoryBundle)?,
+        )? {
+            RulesEvaluation::Exclude
+            | RulesEvaluation::Omit
+            | RulesEvaluation::NoRule
+            | RulesEvaluation::SealSymlink(..)
+            | RulesEvaluation::SealNested(..) => Ok(()),
+            RulesEvaluation::SealRegularFile(relative_path, optional) => {
+                let data = std::fs::read(file.absolute_path())?;
+
+                self.resources
+                    .seal_regular_file(FilesFlavor::Rules, relative_path, data, optional)
+            }
+        }
+    }
+
+    /// Process a file for resource handling.
+    ///
+    /// This determines whether a file is relevant for inclusion in the CodeResources
+    /// file and takes actions to process it, if necessary.
+    pub fn process_file(
+        &mut self,
+        file: &DirectoryBundleFile,
+        file_handler: &dyn BundleFileHandler,
+    ) -> Result<(), AppleCodesignError> {
+        self.process_file_rules2(file, file_handler)?;
+        self.process_file_rules(file)
+    }
+
+    /// Process a nested bundle for inclusion in resource handling.
+    ///
+    /// This will attempt to seal the main digest of the bundle into this resources file.
+    pub fn process_nested_bundle(
+        &mut self,
+        relative_path: &str,
+        bundle: &DirectoryBundle,
+    ) -> Result<(), AppleCodesignError> {
+        let main_exe = match bundle
+            .files(false)
+            .map_err(AppleCodesignError::DirectoryBundle)?
+            .into_iter()
+            .find(|file| matches!(file.is_main_executable(), Ok(true)))
+        {
+            Some(path) => path,
+            None => {
+                warn!(
+                    "nested bundle at {} does not have main executable; nothing to seal",
+                    relative_path
+                );
                 return Ok(());
             }
         };
 
-        if let Some(target) = file
-            .symlink_target()
-            .map_err(AppleCodesignError::DirectoryBundle)?
-        {
-            let target = target.to_string_lossy().replace('\\', "/");
+        let (relative_path, optional) =
+            match Self::evaluate_rules(&self.rules2, relative_path, None)? {
+                RulesEvaluation::SealRegularFile(relative_path, optional) => {
+                    (relative_path, optional)
+                }
+                RulesEvaluation::SealNested(relative_path, optional) => (relative_path, optional),
+                RulesEvaluation::Exclude => {
+                    info!(
+                        "excluding signing nested bundle {} because of matched resources rule",
+                        relative_path
+                    );
+                    return Ok(());
+                }
+                res => {
+                    warn!(
+                        "unexpected resource rules evaluation result for nested bundle {}: {:?}",
+                        relative_path, res
+                    );
+                    return Err(AppleCodesignError::BundleUnexpectedResourceRuleResult);
+                }
+            };
 
-            info!("sealing symlink {} -> {}", relative_path, target);
-            self.resources.seal_symlink(relative_path, target);
-            file_handler.install_file(file)?;
-        } else {
-            let data = std::fs::read(file.absolute_path())?;
+        let macho_data = std::fs::read(main_exe.absolute_path())?;
+        let macho_info = SignedMachOInfo::parse_data(&macho_data)?;
 
-            // If nested bit is set, treat as Mach-O binary to be signed.
-            if rule.nested {
-                let macho_info = file_handler.sign_and_install_macho(file)?;
-                info!("sealing Mach-O file {}", relative_path);
-                self.resources
-                    .seal_macho(relative_path, &macho_info, rule.optional)?;
-            } else {
-                info!("sealing regular file {}", relative_path);
-                self.resources
-                    .seal_regular_file(relative_path, data, rule.optional)?;
-                file_handler.install_file(file)?;
-            }
-        }
+        info!("sealing nested bundle at {}", relative_path);
+        self.resources
+            .seal_macho(relative_path, &macho_info, optional)?;
 
         Ok(())
     }
 
-    /// Add metadata for an additional signed Mach-O file.
-    ///
-    /// This is likely used to register the metadata of a nested bundle. The
-    /// metadata likely comes from the first Mach-O binary in the nested bundle's
-    /// main executable.
-    pub fn add_signed_macho_file(
-        &mut self,
-        path: &str,
-        info: &SignedMachOInfo,
-    ) -> Result<(), AppleCodesignError> {
-        self.resources.seal_macho(path, info, false)
-    }
-
     /// Write CodeResources XML content to a writer.
     pub fn write_code_resources(&self, writer: impl Write) -> Result<(), AppleCodesignError> {
         self.resources.to_writer_xml(writer)
