@@ -43,6 +43,12 @@ pub enum AppleCodesignError {
     #[error("JSON serialization error: {0}")]
     SerdeJson(#[from] serde_json::Error),
 
+    #[error("YAML serialization error: {0}")]
+    SerdeYaml(#[from] serde_yaml::Error),
+
+    #[error("glob error: {0}")]
+    GlobPattern(#[from] glob::PatternError),
+
     #[error("problems reported during verification")]
     VerificationProblems,
 
@@ -70,6 +76,12 @@ pub enum AppleCodesignError {
     #[error("JWT error: {0}")]
     Jwt(#[from] jsonwebtoken::errors::Error),
 
+    #[error("XAR error: {0}")]
+    Xar(#[from] apple_xar::Error),
+
+    #[error("Apple flat package error: {0}")]
+    FlatPackage(#[from] apple_flat_package::Error),
+
     #[error("unable to locate __TEXT segment")]
     MissingText,
 
@@ -100,6 +112,9 @@ pub enum AppleCodesignError {
     #[error("SuperBlob data is malformed")]
     SuperblobMalformed,
 
+    #[error("specified path is not of a recognized type")]
+    UnrecognizedPathType,
+
     #[error("functionality not implemented: {0}")]
     Unimplemented(&'static str),
 
@@ -154,6 +169,9 @@ pub enum AppleCodesignError {
     #[error("signature data too large (please report this issue)")]
     SignatureDataTooLarge,
 
+    #[error("invalid builder operation: {0}")]
+    SignatureBuilder(&'static str),
+
     #[error("HTTP error: {0}")]
     Reqwest(#[from] reqwest::Error),
 
@@ -178,6 +196,11 @@ pub enum AppleCodesignError {
     #[error("bundle Info.plist does not define CFBundleExecutable: {0}")]
     BundleNoMainExecutable(PathBuf),
 
+    #[error(
+        "unexpected resource rule evaluation when signing nested bundle (please report this issue)"
+    )]
+    BundleUnexpectedResourceRuleResult,
+
     #[error("unable to parse settings scope: {0}")]
     ParseSettingsScope(String),
 
@@ -227,6 +250,12 @@ pub enum AppleCodesignError {
     #[error("error writing app metadata XML: {0}")]
     AppMetadataXml(xml::writer::Error),
 
+    #[error("error writing XML: {0}")]
+    XmlWrite(xml::writer::Error),
+
+    #[error("signing XAR archives requires a signing certificate")]
+    XarNoAdhoc,
+
     #[error("Apple Transporter executable not found")]
     TransporterNotFound,
 
@@ -236,6 +265,9 @@ pub enum AppleCodesignError {
     #[error("do not know how to notarize {0}")]
     NotarizeUnsupportedPath(PathBuf),
 
+    #[error("error parsing metadata from flat package")]
+    NotarizeFlatPackageParse,
+
     #[error("failure when performing transporter upload (see logs)")]
     NotarizeUploadFailure,
 
@@ -275,9 +307,37 @@ pub enum AppleCodesignError {
     #[error("do not support stapling {0:?} bundles")]
     StapleUnsupportedBundleType(apple_bundles::BundlePackageType),
 
+    #[error("XAR file is malformed; cannot staple")]
+    StapleMalformedXar,
+
     #[error("failed to find main executable in bundle")]
     StapleMainExecutableNotFound,
 
     #[error("do not know how to staple {0}")]
     StapleUnsupportedPath(PathBuf),
+
+    #[error("bad header magic in DMG; not a DMG file?")]
+    DmgBadMagic,
+
+    #[error("cannot notarize DMG without an embedded signature")]
+    DmgNotarizeNoSignature,
+
+    #[error("cannot staple DMG without an embedded signature")]
+    DmgStapleNoSignature,
+
+    #[error("failed to find certificate in smartcard slot {0}")]
+    SmartcardNoCertificate(String),
+
+    #[error("failed to authenticate with smartcard device")]
+    SmartcardFailedAuthentication,
+
+    #[cfg(feature = "yubikey")]
+    #[error("YubiKey error: {0}")]
+    YubiKey(#[from] yubikey::Error),
+
+    #[error("poisoned lock")]
+    PoisonedLock,
+
+    #[error("internal API / logic error: {0}")]
+    LogicError(String),
 }
