@@ -41,15 +41,14 @@
 //!   Mach-O binaries outside the main executable will also be signed. Non
 //!   Mach-O/code files will be digested. A `CodeResources` XML file will be
 //!   produced.
-//! * Submit notarization requests to Apple and query notarization status. (Only
-//!   macOS `.app` bundles are currently supported.)
-//! * Retrieve notarization tickets from Apple and staple. (Only bundles are currently
-//!   supported.)
+//! * Submit notarization requests to Apple and query notarization status. (Bundles,
+//!   DMGs, and `.pkg` installers are all supported.)
+//! * Retrieve notarization tickets from Apple and staple. All formats supporting
+//!   notarization can be stapled.
 //!
 //! There are a number of missing features and capabilities from this crate
 //! that we hope are eventually implemented:
 //!
-//! * Only embedded signatures are supported. (No support for detached signatures.)
 //! * No parsing of the Code Signing Requirements DSL. We support parsing the binary
 //!   requirements to Rust structs, serializing back to binary, and rendering to the
 //!   human friendly DSL. You will need to use the `csreq` tool to compile an
@@ -62,8 +61,8 @@
 //!   based stores (such as Yubikeys). We also don't look for necessary X.509
 //!   certificate extensions that Apple's verification likely mandates, which we should
 //!   do and enforce.
-//! * Not all signable formats can be notarized. Support for `.dmg` files is a major
-//!   limitation.
+//! * Some more advanced bundles or `.pkg` files may not sign, notarize, or staple
+//!   correctly. Problems here are considered bugs and should be reported.
 //!
 //! There is missing features and functionality that will likely never be implemented:
 //!
@@ -78,25 +77,27 @@
 //!
 //! # End-User Documentation
 //!
-//! See [tutorial] for end-user documentation showing how this crate can be used.
+//! The end-user documentation is maintained as a Sphinx docs tree in the `docs`
+//! directory. The latest version of the documentation is published at
+//! <https://pyoxidizer.readthedocs.io/en/latest/apple_codesign.html>.
 //!
 //! # Getting Started
 //!
-//! This crate is still in early phases of development. Until things are more mature,
-//! a good place to start with the source code is `main.rs` to get a feel for what
-//! CLI commands do.
+//! The [UnifiedSigner] type is a good place to start to see how the high level API for
+//! signing is implemented.
 //!
-//! The [MachOSigner] type is your gateway to how code signing
-//! is performed.
+//! To learn about the low-level data structures in embedded code signatures, read
+//! [specification]. Or look at the code in [embedded_signature] and
+//! [embedded_signature_builder].
 //!
-//! The [AppleSignable] trait extends the [goblin::mach::MachO] type with code
-//! signing functionality.
+//! [MachOSigner] is the type responsible for signing Mach-O files.
 //!
-//! The [EmbeddedSignature] type describes existing code signatures on Mach-O
-//! binaries.
+//! [BundleSigner] is the type responsible for signing bundles.
 //!
-//! If you'd like to learn about the technical underpinnings of code signing on Apple
-//! platforms, see [specification].
+//! [dmg::DmgSigner] signs DMG files.
+//!
+//! The [EmbeddedSignature] represents a parsed Apple code signature and provides API
+//! for data retrieval.
 //!
 //! # Accessing Apple Code Signing Certificates
 //!
@@ -125,6 +126,11 @@ pub mod code_requirement;
 pub use code_requirement::*;
 mod code_resources;
 pub use code_resources::*;
+pub mod dmg;
+pub mod embedded_signature;
+pub use embedded_signature::*;
+pub mod embedded_signature_builder;
+pub use embedded_signature_builder::*;
 pub mod entitlements;
 mod error;
 pub use error::*;
@@ -141,11 +147,16 @@ pub mod notarization;
 pub use notarization::*;
 mod policy;
 pub use policy::*;
+mod reader;
+pub use reader::*;
+mod signing_settings;
+pub use signing_settings::*;
 mod signing;
 pub use signing::*;
 pub mod specification;
 pub mod stapling;
 pub mod ticket_lookup;
-pub mod tutorial;
 mod verify;
 pub use verify::*;
+#[cfg(feature = "yubikey")]
+pub mod yubikey;
