@@ -2,30 +2,30 @@
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at https://mozilla.org/MPL/2.0/.
 
-//! Signing binaries.
+//! Signing mach-o binaries.
 //!
-//! This module contains code for signing binaries.
+//! This module contains code for signing mach-o binaries.
 
 use {
     crate::{
         code_directory::{CodeDirectoryBlob, CodeSignatureFlags},
         code_hash::compute_code_hashes,
-        code_requirement::{CodeRequirementExpression, CodeRequirements},
+        code_requirement::{CodeRequirementExpression, CodeRequirements, RequirementType},
+        embedded_signature::{
+            BlobData, CodeSigningSlot, Digest, EmbeddedSignature, EntitlementsBlob,
+            EntitlementsDerBlob, RequirementSetBlob,
+        },
+        embedded_signature_builder::EmbeddedSignatureBuilder,
         entitlements::plist_to_executable_segment_flags,
         error::AppleCodesignError,
         macho::{
-            create_superblob, find_macho_targeting, parse_version_nibbles,
-            semver_to_macho_target_version, AppleSignable, Blob, BlobWrapperBlob, CodeSigningMagic,
-            CodeSigningSlot, Digest, DigestType, EmbeddedSignature, EntitlementsBlob,
-            EntitlementsDerBlob, RequirementSetBlob, RequirementType,
+            find_macho_targeting, iter_macho, parse_version_nibbles,
+            semver_to_macho_target_version, AppleSignable,
         },
         policy::derive_designated_requirements,
-        signing::{DesignatedRequirementMode, SettingsScope, SigningSettings},
+        signing_settings::{DesignatedRequirementMode, SettingsScope, SigningSettings},
         ExecutableSegmentFlags,
     },
-    bcder::{encode::PrimitiveContent, Oid},
-    bytes::Bytes,
-    cryptographic_message_syntax::{asn1::rfc5652::OID_ID_DATA, SignedDataBuilder, SignerBuilder},
     goblin::mach::{
         constants::{SEG_LINKEDIT, SEG_PAGEZERO},
         load_command::{
@@ -38,53 +38,8 @@ use {
     scroll::{ctx::SizeWith, IOwrite},
     std::{borrow::Cow, cmp::Ordering, collections::HashMap, io::Write},
     tugger_apple::create_universal_macho,
-    x509_certificate::{rfc5652::AttributeValue, DigestAlgorithm},
 };
 
-/// OID for signed attribute containing plist of code directory hashes.
-///
-/// 1.2.840.113635.100.9.1.
-const CDHASH_PLIST_OID: bcder::ConstOid = Oid(&[42, 134, 72, 134, 247, 99, 100, 9, 1]);
-
-/// OID for signed attribute containing the SHA-256 of code directory digests.
-///
-/// 1.2.840.113635.100.9.2
-const CDHASH_SHA256_OID: bcder::ConstOid = Oid(&[42, 134, 72, 134, 247, 99, 100, 9, 2]);
-
-/// Obtain the XML plist containing code directory hashes.
-///
-/// This plist is embedded as a signed attribute in the CMS signature.
-pub fn create_code_directory_hashes_plist<'a>(
-    code_directories: impl Iterator<Item = &'a CodeDirectoryBlob<'a>>,
-    digest_type: DigestType,
-) -> Result<Vec<u8>, AppleCodesignError> {
-    let hashes = code_directories
-        .map(|cd| {
-            let mut digest = cd.digest_with(digest_type)?;
-
-            // While we may use stronger digests, it appears that the XML in the
-            // signed attribute is always truncated so it is the length of a SHA-1
-            // digest.
-            digest.truncate(20);
-
-            Ok(plist::Value::Data(digest))
-        })
-        .collect::<Result<Vec<_>, AppleCodesignError>>()?;
-
-    let mut plist = plist::Dictionary::new();
-    plist.insert("cdhashes".to_string(), plist::Value::Array(hashes));
-
-    let mut buffer = Vec::<u8>::new();
-    plist::Value::from(plist)
-        .to_writer_xml(&mut buffer)
-        .map_err(AppleCodesignError::CodeDirectoryPlist)?;
-    // We also need to include a trailing newline to conform with Apple's XML
-    // writer.
-    buffer.push(b'\n');
-
-    Ok(buffer)
-}
-
 /// Derive a new Mach-O binary with new signature data.
 fn create_macho_with_signature(
     macho_data: &[u8],
@@ -283,27 +238,7 @@ impl<'data> MachOSigner<'data> {
     /// The data will be parsed as a Mach-O binary (either single arch or fat/universal)
     /// and validated that we are capable of signing it.
     pub fn new(macho_data: &'data [u8]) -> Result<Self, AppleCodesignError> {
-        let mach = Mach::parse(macho_data)?;
-
-        let machos = match mach {
-            Mach::Binary(macho) => {
-                macho.check_signing_capability()?;
-
-                vec![macho]
-            }
-            Mach::Fat(multiarch) => {
-                let mut machos = vec![];
-
-                for index in 0..multiarch.narches {
-                    let macho = multiarch.get(index)?;
-                    macho.check_signing_capability()?;
-
-                    machos.push(macho);
-                }
-
-                machos
-            }
-        };
+        let machos = iter_macho(macho_data)?.collect::<Vec<_>>();
 
         Ok(Self { macho_data, machos })
     }
@@ -414,108 +349,50 @@ impl<'data> MachOSigner<'data> {
         macho: &MachO,
         previous_signature: Option<&EmbeddedSignature>,
     ) -> Result<Vec<u8>, AppleCodesignError> {
+        let mut builder = EmbeddedSignatureBuilder::default();
+
+        for (slot, blob) in self.create_special_blobs(settings, previous_signature)? {
+            builder.add_blob(slot, blob)?;
+        }
+
         let code_directory =
             self.create_code_directory(settings, macho_data, macho, previous_signature)?;
         info!("code directory version: {}", code_directory.version);
 
-        // By convention, the Code Directory goes first.
-        let mut blobs = vec![(
-            CodeSigningSlot::CodeDirectory,
-            code_directory.to_blob_bytes()?,
-        )];
-        blobs.extend(self.create_special_blobs(settings, previous_signature)?);
+        builder.add_code_directory(CodeSigningSlot::CodeDirectory, code_directory)?;
 
-        // And the CMS signature goes last.
-        if settings.signing_key().is_some() {
-            blobs.push((
-                CodeSigningSlot::Signature,
-                BlobWrapperBlob::from_data(&self.create_cms_signature(settings, &code_directory)?)
-                    .to_blob_bytes()?,
-            ));
+        if let Some(digests) = settings.extra_digests(SettingsScope::Main) {
+            for digest_type in digests {
+                // Since everything consults settings for the digest to use, just make a new settings
+                // with a different digest.
+                let mut alt_settings = settings.clone();
+                alt_settings.set_digest_type(*digest_type);
+
+                info!(
+                    "adding alternative code directory using digest {:?}",
+                    digest_type
+                );
+                let cd = self.create_code_directory(
+                    &alt_settings,
+                    macho_data,
+                    macho,
+                    previous_signature,
+                )?;
+
+                builder.add_alternative_code_directory(cd.into())?;
+            }
         }
 
-        create_superblob(CodeSigningMagic::EmbeddedSignature, blobs.iter())
-    }
-
-    /// Create a CMS `SignedData` structure containing a cryptographic signature.
-    ///
-    /// This becomes the content of the `EmbeddedSignature` blob in the `Signature` slot.
-    ///
-    /// This function will error if a signing key has not been specified.
-    ///
-    /// This takes an explicit Mach-O to operate on due to a circular dependency
-    /// between writing out the Mach-O and digesting its content. See the note
-    /// in [MachOSigner] for details.
-    pub fn create_cms_signature(
-        &self,
-        settings: &SigningSettings,
-        code_directory: &CodeDirectoryBlob,
-    ) -> Result<Vec<u8>, AppleCodesignError> {
-        let (signing_key, signing_cert) = settings
-            .signing_key()
-            .ok_or(AppleCodesignError::NoSigningCertificate)?;
-
-        if let Some(cn) = signing_cert.subject_common_name() {
-            warn!("creating cryptographic signature with certificate {}", cn);
+        if let Some((signing_key, signing_cert)) = settings.signing_key() {
+            builder.create_cms_signature(
+                signing_key,
+                signing_cert,
+                settings.time_stamp_url(),
+                settings.certificate_chain().iter().cloned(),
+            )?;
         }
 
-        // We need the blob serialized content of the code directory to compute
-        // the message digest using alternate data.
-        let code_directory_raw = code_directory.to_blob_bytes()?;
-
-        // We need an XML plist containing code directory hashes to include as a signed
-        // attribute.
-        let code_directories = vec![code_directory];
-        let code_directory_hashes_plist = create_code_directory_hashes_plist(
-            code_directories.into_iter(),
-            code_directory.hash_type,
-        )?;
-
-        let signer = SignerBuilder::new(signing_key, signing_cert.clone())
-            .message_id_content(code_directory_raw)
-            .signed_attribute_octet_string(
-                Oid(Bytes::copy_from_slice(CDHASH_PLIST_OID.as_ref())),
-                &code_directory_hashes_plist,
-            );
-
-        // If we're using a digest beyond SHA-1, that digest is included as an additional
-        // signed attribute. However, Apple is using unregistered OIDs here. We only know about
-        // the SHA-256 one. It exists as an `(OID, OCTET STRING)` value where the OID
-        // is 2.16.840.1.101.3.4.2.1, which is registered.
-        let signer = if code_directory.hash_type == DigestType::Sha256 {
-            let digest = code_directory.digest_with(DigestType::Sha256)?;
-
-            signer.signed_attribute(
-                Oid(CDHASH_SHA256_OID.as_ref().into()),
-                vec![AttributeValue::new(bcder::Captured::from_values(
-                    bcder::Mode::Der,
-                    bcder::encode::sequence((
-                        Oid::from(DigestAlgorithm::Sha256).encode_ref(),
-                        bcder::OctetString::new(digest.into()).encode_ref(),
-                    )),
-                ))],
-            )
-        } else {
-            signer
-        };
-
-        let signer = if let Some(time_stamp_url) = settings.time_stamp_url() {
-            info!("Using time-stamp server {}", time_stamp_url);
-            signer.time_stamp_url(time_stamp_url.clone())?
-        } else {
-            signer
-        };
-
-        let der = SignedDataBuilder::default()
-            // The default is `signed-data`. But Apple appears to use the `data` content-type,
-            // in violation of RFC 5652 Section 5, which says `signed-data` should be
-            // used when there are signatures.
-            .content_type(Oid(OID_ID_DATA.as_ref().into()))
-            .signer(signer)
-            .certificates(settings.certificate_chain().iter().cloned())
-            .build_der()?;
-
-        Ok(der)
+        builder.create_superblob()
     }
 
     /// Attempt to resolve the binary identifier to use.
@@ -566,8 +443,11 @@ impl<'data> MachOSigner<'data> {
             );
         }
 
-        let previous_cd =
-            previous_signature.and_then(|signature| signature.code_directory().unwrap_or(None));
+        let previous_cd = previous_signature.and_then(|signature| {
+            signature
+                .code_directory_for_digest(*settings.digest_type())
+                .unwrap_or(None)
+        });
 
         let mut flags = CodeSignatureFlags::empty();
 
@@ -659,6 +539,10 @@ impl<'data> MachOSigner<'data> {
             }
         }
 
+        if let Some(flags) = exec_seg_flags {
+            info!("final executable segment flags: {:?}", flags);
+        }
+
         // The runtime version is the SDK version from the targeting loader commands. Same
         // u32 with nibbles encoding the version.
         //
@@ -713,18 +597,7 @@ impl<'data> MachOSigner<'data> {
                 .map(|v| Digest { data: v.into() })
                 .collect::<Vec<_>>();
 
-        let mut special_hashes = self
-            .create_special_blobs(settings, previous_signature)?
-            .into_iter()
-            .map(|(slot, data)| {
-                Ok((
-                    slot,
-                    Digest {
-                        data: settings.digest_type().digest(&data)?.into(),
-                    },
-                ))
-            })
-            .collect::<Result<HashMap<_, _>, AppleCodesignError>>()?;
+        let mut special_hashes = HashMap::new();
 
         // There is no corresponding blob for the info plist data since it is provided
         // externally to the embedded signature.
@@ -733,13 +606,13 @@ impl<'data> MachOSigner<'data> {
                 special_hashes.insert(
                     CodeSigningSlot::Info,
                     Digest {
-                        data: settings.digest_type().digest(data)?.into(),
+                        data: settings.digest_type().digest_data(data)?.into(),
                     },
                 );
             }
             None => {
                 if let Some(previous_cd) = &previous_cd {
-                    if let Some(digest) = previous_cd.special_hashes.get(&CodeSigningSlot::Info) {
+                    if let Some(digest) = previous_cd.slot_digest(CodeSigningSlot::Info) {
                         if !digest.is_null() {
                             special_hashes.insert(CodeSigningSlot::Info, digest.to_owned());
                         }
@@ -755,17 +628,14 @@ impl<'data> MachOSigner<'data> {
                 special_hashes.insert(
                     CodeSigningSlot::ResourceDir,
                     Digest {
-                        data: settings.digest_type().digest(data)?.into(),
+                        data: settings.digest_type().digest_data(data)?.into(),
                     }
                     .to_owned(),
                 );
             }
             None => {
                 if let Some(previous_cd) = &previous_cd {
-                    if let Some(digest) = previous_cd
-                        .special_hashes
-                        .get(&CodeSigningSlot::ResourceDir)
-                    {
+                    if let Some(digest) = previous_cd.slot_digest(CodeSigningSlot::ResourceDir) {
                         if !digest.is_null() {
                             special_hashes.insert(CodeSigningSlot::ResourceDir, digest.to_owned());
                         }
@@ -791,33 +661,27 @@ impl<'data> MachOSigner<'data> {
         };
 
         let mut cd = CodeDirectoryBlob {
-            version: 0,
             flags,
             code_limit,
-            hash_size: settings.digest_type().hash_len()? as u8,
-            hash_type: *settings.digest_type(),
+            digest_size: settings.digest_type().hash_len()? as u8,
+            digest_type: *settings.digest_type(),
             platform,
             page_size,
-            spare2: 0,
-            scatter_offset: None,
-            spare3: None,
             code_limit_64,
             exec_seg_base,
             exec_seg_limit,
             exec_seg_flags,
             runtime,
-            pre_encrypt_offset: None,
-            linkage_hash_type: None,
-            linkage_truncated: None,
-            spare4: None,
-            linkage_offset: None,
-            linkage_size: None,
             ident,
             team_name,
-            code_hashes,
-            special_hashes,
+            code_digests: code_hashes,
+            ..Default::default()
         };
 
+        for (slot, digest) in special_hashes {
+            cd.set_slot_digest(slot, digest)?;
+        }
+
         cd.adjust_version(target);
         cd.clear_newer_fields();
 
@@ -835,7 +699,7 @@ impl<'data> MachOSigner<'data> {
         &self,
         settings: &SigningSettings,
         previous_signature: Option<&EmbeddedSignature>,
-    ) -> Result<Vec<(CodeSigningSlot, Vec<u8>)>, AppleCodesignError> {
+    ) -> Result<Vec<(CodeSigningSlot, BlobData<'static>)>, AppleCodesignError> {
         let mut res = Vec::new();
 
         let mut requirements = CodeRequirements::default();
@@ -868,14 +732,14 @@ impl<'data> MachOSigner<'data> {
             let mut blob = RequirementSetBlob::default();
             requirements.add_to_requirement_set(&mut blob, RequirementType::Designated)?;
 
-            res.push((CodeSigningSlot::RequirementSet, blob.to_blob_bytes()?));
+            res.push((CodeSigningSlot::RequirementSet, blob.into()));
         }
 
         if let Some(entitlements) = settings.entitlements_xml(SettingsScope::Main)? {
             info!("adding entitlements XML");
             let blob = EntitlementsBlob::from_string(&entitlements);
 
-            res.push((CodeSigningSlot::Entitlements, blob.to_blob_bytes()?));
+            res.push((CodeSigningSlot::Entitlements, blob.into()));
         }
 
         // The DER encoded entitlements weren't always present in the signature. The feature
@@ -887,7 +751,7 @@ impl<'data> MachOSigner<'data> {
             info!("adding entitlements DER");
             let blob = EntitlementsDerBlob::from_plist(value)?;
 
-            res.push((CodeSigningSlot::EntitlementsDer, blob.to_blob_bytes()?));
+            res.push((CodeSigningSlot::EntitlementsDer, blob.into()));
         }
 
         Ok(res)
